#include "loadelt.h"
#include "globalt.h"
#include "aliast.h"
#include "globmatt.h"
#include <string.h>

loadelt::loadelt()
{
  //  loaded element id (number of loaded element)
  eid=-1;
  //  number of boundary objects (end nodes, edges, surfaces)
  nbo=0;
  //  number of nodes on boundary object
  nnbo=0;
  
  //  boundary conditions (for nongrowing problems)
  bc=NULL;
  //  boundary conditions (for growing problems)
  bcf=NULL;
  
  //  id of nodal values or climatic conditions (for nongrowing problems)
  nvid=NULL;
  //  id of nodal values or climatic conditions (for growing problems)
  nvidf=NULL;
  //  id of nodal values describing transmission coefficients (for nongrowing problems)
  trcid=NULL;
  //  id of nodal values describing transmission coefficients (for growing problems)
  trcidf=NULL;
  //  id of nodal values describing transmission/radiation coefficients (for nongrowing problems)
  trrid=NULL;
  //  id of nodal values describing transmission/radiation coefficients (for growing problems)
  trridf=NULL;
}

loadelt::~loadelt()
{
  delete [] bc;
  delete [] bcf;
  
  delete [] nvid;
  delete [] nvidf;
  delete [] trcid;
  delete [] trcidf;
  delete [] trrid;
  delete [] trridf;
}

/**
   function reads boundary conditions on elements
   
   @param in - pointer to input file
   @param lcid - load case id
*/
void loadelt::read (XFILE *in,long lcid)
{
  long i;
  
  //  element id
  xfscanf (in,"%ld",&eid);
  eid--;
  
  //  check of element id
  if (eid > Tt->ne-1){
    print_err("Element number in boundary conditions is greater than total number of elements", __FILE__, __LINE__, __func__);
    abort();
  }
  if (eid < 0){
    print_err("Element number in boundary conditions is less than 0", __FILE__, __LINE__, __func__);
    abort();
  }
  
  //  number of boundary objects (end nodes, edges, surfaces)
  //  number of nodes on each boundary object
  Tt->give_nbobjects (eid,nbo,nnbo);
  
  if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
    //  indicator of boundary conditions
    bcf = new gfunct [nbo];
    //  id of nodal values or climatic conditions
    nvidf = new gfunct [nbo];
    //  id of nodal values describing transmission coefficients
    trcidf = new gfunct [nbo];
    //  id of nodal values describing transmission/radiation coefficients
    trridf = new gfunct [nbo];
  }
  else{
    //  indicators of boundary conditions
    bc = new long [nbo];
    //  id of nodal values or climatic conditions
    nvid = new long [nbo];
    //  id of nodal values describing transmission coefficients
    trcid = new long [nbo];
    //  id of nodal values describing transmission/radiation coefficients
    trrid = new long [nbo];
  }
  
  
  //  loop over boundary objects (end nodes, edges or surfaces)
  for (i=0;i<nbo;i++){

    if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){	
      //  type of boundary condition
      bcf[i].read (in);
      //  id of nodal values
      nvidf[i].read (in);
      //  id of transmission coefficients
      trcidf[i].read (in);
      //  id of transmission/radiation coefficients
      trridf[i].read (in);
    }
    else{
      //  type of boundary condition
      xfscanf (in,"%ld",bc+i);
      
      nvid[i]=-1;
      trcid[i]=-1;
      trrid[i]=-1;
      
      if (bc[i]>10){
	//  prescribed transmission (Newton boundary conditon)
	
	//  id of exterior prescribed values
	xfscanf (in,"%ld",nvid+i);
	nvid[i]--;
	//  id of transmission coefficients
	xfscanf (in,"%ld",trcid+i);
	trcid[i]--;
	//  id of transmission/radiation coefficients
	xfscanf (in,"%ld",trrid+i);
	trrid[i]--;
      }
      if (bc[i]==3 || bc[i]==4){
	// prescribed detail climatic conditions
	
	//  id of detail climatic conditions
	xfscanf (in,"%ld",nvid+i);
	nvid[i]--;
      }
    }
  }
  
  //  determination of auxiliary indicators on elements
  //  they serve for efficient computation of fluxes
  //  only elements with appropriate transi indicators are performed
  for (i=0;i<nbo;i++){
    if (bc[i]==2 || bc[i]==3 || bc[i]==4){
      if (Tt->elements[eid].transi[lcid]<2)
	Tt->elements[eid].transi[lcid]=2;
    }
    if (bc[i]==4 || bc[i]>10){
      if (Tt->elements[eid].transi[lcid]<2)
	Tt->elements[eid].transi[lcid]=3;
      else
	Tt->elements[eid].transi[lcid]=4;
    }
  }
  
}


/**
   function prints boundary conditions on elements
   
   @param out - pointer to output file
   @param lcid - load case id
*/
void loadelt::print (FILE *out,long lcid)
{
  long i;
  
  //  element id
  fprintf (out,"\n\n %ld ",eid+1);
  
  //  loop over boundary objects (end nodes, edges or surfaces)
  for (i=0;i<nbo;i++){
    if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
      //  type of boundary condition
      bcf[i].print (out);
      //  id of nodal values
      nvidf[i].print (out);
      //  id of transmission coefficients
      trcidf[i].print (out);
      //  id of transmission/radiation coefficients
      trridf[i].print (out);
    }
    else{
      fprintf (out,"\n %ld",bc[i]);
      
      if (bc[i]>10){
	//  prescribed transmission (Newton boundary conditon)
	
	//  id of exterior prescribed values
	fprintf (out," %ld",nvid[i]);
	//  id of transmission coefficients
	fprintf (out," %ld",trcid[i]);
	//  id of transmission/radiation coefficients
	fprintf (out," %ld",trrid[i]);
      }
      if (bc[i]==3 || bc[i]==4){
	// prescribed detail climatic conditions
	
	//  id of detail climatic conditions
	fprintf (out," %ld",nvid[i]);
      }
      
    }
  }
  
}


/**
   function assembles indicators of boundary conditions
   
   @param bc - array of indicators of boundary conditions
   
   JK, 24.11.2008
*/
void loadelt::give_bc (long *av)
{
  long i;
  
  //  loop over boundary objects - end nodes, edges or surfaces
  for (i=0;i<nbo;i++){
    av[i]=bc[i];
  }
}

/**
   function assembles nodal values defined on boundaries
   there are values for all boundary objects on elements
   
   @param lcid - load case id
   @param fe - array of actual nodal values
   
   JK, 24.11.2008
*/
void loadelt::give_nodval (long lcid,vector &fe)
{
  long i,j,k,l,m,ndofn,*ncn,*bnodes,ipp;
  double *nval;
  vector av;
  
  //  all components are set to zero
  fillv (0.0,fe);
  
  //  first element material point
  ipp=Tt->elements[eid].ipp[0][0];
  //  array of node numbers on boundary object
  bnodes = new long [nnbo];
  
  
  k=0;
  //  loop over boundary objects - end nodes, edges or surfaces
  for (i=0;i<nbo;i++){
    if (bc[i]==0){
      k+=nnbo;
    }

    if (bc[i]==2 || bc[i]>10){
      //  prescribed boundary fluxes (Neumann boundary condition)
      //  or prescribed external vaules (Newton boundary condition)
      
      //  id of nodal values
      l=nvid[i];
      
      if (l<0)
	print_err("negative id of nodal values",__FILE__,__LINE__,__func__);
      
      allocv (nnbo,av);
      
      //  nodal values for required boundary object
      Tb->lc[lcid].nodval[l].give_val (Tp->time,av);
      
      for (j=0;j<nnbo;j++){
	fe[k]=av[j];
	k++;
      }
      destrv (av);
    }
    
    
    
    
    if (bc[i]==3){
      /* *************************************************************** */
      /*  computation of boundary fluxes from climate condition records  */
      /* *************************************************************** */
      
      //  id of nodal values
      l=nvid[i];
      
      if (l<0)
	print_err("negative id of nodal values",__FILE__,__LINE__,__func__);
      
      if(l > Tb->lc[lcid].ncc-1){
	print_err("id of climatic conditions is greater than the number of clim. conditions", __FILE__, __LINE__, __func__);
	abort ();
      }
      
      //  determination of nodes on required boundary object (end node, edge, surface)
      Tt->give_bonodes (eid,i,bnodes);
      
      //  loop over boundary nodes
      for (j=0;j<nnbo;j++){
	//  number of DOFs of node
	ndofn = Gtt->give_ndofn (bnodes[j]);
	//  nodal code numbers
	ncn = new long [ndofn];
	//  nodal values of computed quantities
	nval = new double [ndofn];
	
	//  code numbers on nodes on boundary objects
	Gtt->give_node_code_numbers (bnodes[j],ncn);
	
	for (m=0;m<ndofn;m++){
	  if (ncn[m]<1)  continue;
	  nval[m]=Lsrst->lhs[ncn[m]-1] + Lsrst->lhsi[ncn[m]-1];
	}
	
	if(nvid[i] < 0)
	  fe[k]=0.0;//for growing_np_problem
	else
	  fe[k]=Tb->lc[lcid].climcond[l].give_flux (lcid,nval,ipp,bnodes[j]);
	
	k++;
	delete [] nval;
	delete [] ncn;
      }
    }
  }
  
  delete [] bnodes;
}

/**
   function assembles nodal values of transmission coefficients
   there are values for all boundary objects on elements
   
   @param lcid - load case id
   @param trc - array of actual transmission coefficients
   
   JK, 24.11.2008
*/
void loadelt::give_trc (long lcid,vector &trc)
{
  long i,j,k,l;
  vector av;
  
  //  all components are set to zero
  fillv (0.0,trc);
  
  k=0;
  //  loop over boundary objects - end nodes, edges or surfaces
  for (i=0;i<nbo;i++){
    
    if (bc[i]==0){
      k+=nnbo;
    }

    if (bc[i]==4 || bc[i]>10){
      //  prescribed boundary fluxes (Neumann boundary condition)
      
      //  id of nodal values
      l=trcid[i];
      
      if (l<0)
	print_err("negative id of transmission coefficients",__FILE__,__LINE__,__func__);
      
      allocv (nnbo,av);
      
      //  nodal values for required boundary object
      Tb->lc[lcid].nodval[l].give_val (Tp->time,av);
      
      for (j=0;j<nnbo;j++){
	trc[k]=av[j];
	k++;
      }
      destrv (av);
    }
    
  }
}

/**
   function assembles nodal values of transmission/radiation coefficients
   there are values for all boundary objects on elements
   
   @param lcid - load case id
   @param trr - array of actual transmission/radiation coefficients
   
   JK, 24.11.2008
*/
void loadelt::give_trr (long lcid,vector &trr)
{
  long i,j,k,l;
  vector av;
  
  //  all components are set to zero
  fillv (0.0,trr);
  
  k=0;
  //  loop over boundary objects - end nodes, edges or surfaces
  for (i=0;i<nbo;i++){
    
    if (bc[i]==0){
      k+=nnbo;
    }

    if (bc[i]==4 || bc[i]>10){
      //  prescribed boundary fluxes (Neumann boundary condition)
      
      //  id of nodal values
      l=trrid[i];
      
      if (l<0)
	print_err("negative id of transmission coefficients",__FILE__,__LINE__,__func__);
      
      allocv (nnbo,av);
      
      //  nodal values for required boundary object
      Tb->lc[lcid].nodval[l].give_val (Tp->time,av);
      
      for (j=0;j<nnbo;j++){
	trr[k]=av[j];
	k++;
      }
      destrv (av);
    }
    
  }
}

