#include "npsolvert.h"
#include "globalt.h"
#include "globmatt.h"
#include "transprint.h"
#include "saddpoint.h"
#include <string.h>

/**
   function solves nonstationary transport problem
   
   function selects a suitable numerical scheme for solution
   
   JK
*/
void solve_nonstationary_problem ()
{
  if (Tp->tprob == discont_nonstat_problem){
    //  formulation based on nodal unknowns is used
    linear_nonstat_solv_dform ();
  }
  else{
    //  formulation based on unknown first nodal derivatives
    //  with respect to time is used
    linear_nonstat_solv_vform ();
  }
}

/**
   function solves linear nonstationary transport problem
   the v-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis
   
   JK, 20.12.2002,
   revised 24.8.2006
*/
void linear_nonstat_solv_vform ()
{
  long i,j,n;
  double dt,end_time,alpha,*d,*p,*lhs,*tdlhs,*rhs,*f;
  
  //  number of unknowns
  n=Ndoft;
  
  //  nodal values
  lhs = Lsrst->give_lhs (0);
  nullv (lhs,n);
  //  time derivatives of nodal values
  tdlhs = Lsrst->give_tdlhs (0);
  nullv (tdlhs,n);
  //  prescribed fluxes / right hand side
  rhs = Lsrst->give_rhs (0);
  nullv (rhs,n);
  
  //  vector of prescribed fluxes (right hand side)
  f = new double [n];
  nullv (f,n);
  //  predictor
  d = new double [n];
  nullv (d,n);
  //  auxiliary vector
  p = new double [n];
  nullv (p,n);
  
  //  coefficient in generalized trapezoidal integration rule
  alpha=Tp->alpha;
  
  //  initial time
  Tp->time=Tp->timecont.starttime ();
  //  end time
  end_time = Tp->timecont.endtime ();
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  //  step number
  i=0;
  
  //  nodes - integration points interpolation
  approximation();  
  
  // pridal Madera 18.1.2008
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();
  
  //  initiation of material models
  Tm->initmaterialmodels();
  
  //  printing of output and graphical informations
  compute_req_valt (0);
  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);
  
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  do{
    
    //  determination of new time instance
    Tp->time=Tp->timecont.newtime ();
    //  computation of backward time step
    dt = Tp->timecont.actualbacktimeincr ();
    
    if (Mesprt != 0)  fprintf (stdout,"\n\n increment number %ld,   time %f,   time step = %f\n",i,Tp->time,dt);
    
    Tm->updateipval ();
    //  update of step number
    i++;
    
    //  capacity matrix    
    capacity_matrix (0);
    
    //  conductivity matrix
    conductivity_matrix (0);
    
    //  predictor
    //  dd = d_n + (1-alpha) dt v_n
    for (j=0;j<n;j++){
      d[j]=lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  auxiliary vector
    //  K dd
    Kmat->gmxv (d,p);
    
    
    //  matrix of the system of equations
    //  C + K alpha dt
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    //  prescribed nodal fluxes f_{n+1}
    trfel_right_hand_side (0,rhs,n);
    
    //  computation of the right hand side vector
    //  f_{n+1} - K dd
    for (j=0;j<n;j++){
      f[j] = rhs[j] - p[j];
    }
    
    //  solution of the system of algebraic equations
    //  (C + alpha dt K) v_{n+1} = f_{n+1} - K dd
    //  time derivatives v_{n+1} are solved
    Tp->ssle->solve_system (Gtt,Kmat,tdlhs,f,Outt);
    
    //  nodal values computed from nodal derivatives
    //  d_{n+1} = dd + alpha dt v_{n+1}
    for (j=0;j<n;j++){
      lhs[j] = d[j] + alpha*dt*tdlhs[j];
    }
    
    solution_correction ();
    compute_req_valt (0);
    //  nodes - integration points interpolation
    approximation ();
    
    // pridal Madera 18.1.2008
    if (Tp->nvs==1 && Tp->pnvs==1)
      actual_previous_nodval ();
    
    
    //  printing of output and graphical informations
    print_stept(0,i,Tp->time,NULL);
    print_flusht();
    
  }while(Tp->time<end_time);
  
  delete [] p;
  delete [] d;
  delete [] f;
  
  print_closet();
}



/**
   function solves linear nonstationary transport problem
   the d-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis

   JK, 23.12.2002,
   revised 24.8.2006
*/
void linear_nonstat_solv_dform ()
{
  long i,j,n;
  double dt,end_time,alpha;
  double *d,*f,*p,*lhs,*tdlhs,*rhs;
  
  //  number of unknowns / degrees of freedom
  n=Ndoft;
  
  //  nodal values
  lhs = Lsrst->give_lhs (0);
  nullv (lhs,n);
  //  time derivatives of nodal values
  tdlhs = Lsrst->give_tdlhs (0);
  nullv (tdlhs,n);
  //  prescribed fluxes / right hand side
  rhs = Lsrst->give_rhs (0);
  nullv (rhs,n);
  
  //  vector of prescribed fluxes (right hand side)
  f = new double [n];
  nullv (f,n);
  //  predictor
  d = new double [n];
  nullv (d,n);
  //  auxiliary vector
  p = new double [n];
  nullv (p,n);


  //  nodes - integration points interpolation
  approximation ();

  // vlozil JM 15.1.2008
  // nulleqother ();
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();
  
  
  //  coefficient in generalized trapezoidal integration rule
  alpha=Tp->alpha;
  
  //  initial time
  Tp->time=Tp->timecont.starttime ();
  //  end time
  end_time = Tp->timecont.endtime ();
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  
  //  step number
  i=0;

  //  initiation of material models
  Tm->initmaterialmodels();

  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);

  // ***************************
  //  main iteration loop  ****
  // ***************************
  do{
    
    //  determination of new time instance
    Tp->time=Tp->timecont.newtime ();
    //  computation of backward time step
    dt = Tp->timecont.actualbacktimeincr ();
    
    if (Mesprt != 0)  fprintf (stdout,"\n\n iteration number %ld, time %f, time step = %f\n",i,Tp->time,dt);
    
    Tm->updateipval ();
    //  update of step number
    i++;
    
    //  capacity matrix
    capacity_matrix (0);
    
    //  conductivity matrix
    conductivity_matrix (0);
    
    //  predictor
    //  dd = d_n + (1-alpha) dt v_n
    for (j=0;j<n;j++){
      d[j]=lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  auxiliary vector
    //  C dd
    Cmat->gmxv (d,p);
    
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    
    //  prescribed nodal fluxes f_{n+1}
    trfel_right_hand_side (0,rhs,n);

    
    //  computation of the right hand side vector
    //  alpha dt f_n+1 + C dd
    for (j=0;j<n;j++){
      f[j] = rhs[j]*alpha*dt + p[j];
    }
    
    //  solution of the system of algebraic equations
    //  (C + alpha dt K) d_{n+1} = alpha dt f_n+1 + C dd
    //  nodal values d_{n+1} are solved
    Tp->ssle->solve_system (Gtt,Kmat,lhs,f,Outt);
    
    //  first time derivatives of nodal values
    // v_{n+1} = 1/alpha/dt (d_{n+1}-dd)
    for (j=0;j<n;j++){
      tdlhs[j]=(lhs[j]-d[j])/dt/alpha;
    }
    
    //  computation of values at integration points from nodal values
    solution_correction ();
    approximation ();
    
    // pridal Madera 18.1.2008
    if (Tp->nvs==1 && Tp->pnvs==1)
      actual_previous_nodval ();
    
    //  printing of output and graphical informations
    print_stept(0,i,Tp->time,NULL);
    print_flusht();
    
  }while(Tp->time<end_time);
  
  delete [] f;
  delete [] p;
  delete [] d;
  
  print_closet();
}
