#include "globalt.h"
#include "saltmat1.h"

/**********************************************
*                                             *
* General material model for 3 media transfer *
*      9 independent parameters               *
***********************************************/
/* x1 (w) is the volumetric water content [m3/m3]
  x2  (Cf) is the concentration of free ions in water [kg/m3]
  Cb the concentration of bonded ions in the whole porous body [kg/m3]
  D the ion diffusivity [m2/s]
  Kapa = the moisture diffusivity [m2/s]
position CORD:     2 - density
                    3 - porosity
                    4 - faktor difusniho odporu
                    5 - kapa
                    6 - sorption izoterm
                    7 - saturated moisture
                    8 - none
                    9 - Cecko
                    10 - Lambda
                    11 - 13 - none
                    14 Dcoef
                    15 - binding isotherm
                    16 - cfmax
                    17 ws
                    18 - 19 - none



*/
saltmat1::saltmat1 ()
{}

saltmat1::~saltmat1 ()
{}




/**
   function computes conductivity matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

*/
void saltmat1::matcond (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;

  switch (n){
  case 1:{
    matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}


/**
   function creates conductivity matrix of the material for 1D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void saltmat1::matcond1d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double w,cf;
  k = 0.0;

  w = Tm->ip[ipp].av[0];
  cf = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = k11(w,cf,0);
  if((ri == 0) && (ci == 1))
    k = k12(w,cf,0);
  if((ri == 1) && (ci == 0))
    k = k21(w,cf,0);
  if((ri == 1) && (ci == 1))
    k = k22(w,cf,0);
  
  d[0][0] = k;
}

/**
   function creates conductivity matrix of the material for 2D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void saltmat1::matcond2d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double w,cf;
  k = 0.0;

  w = Tm->ip[ipp].av[0];
  cf = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = k11(w,cf,0);
  if((ri == 0) && (ci == 1))
    k = k12(w,cf,0);
  if((ri == 1) && (ci == 0))
    k = k21(w,cf,0);
  if((ri == 1) && (ci == 1))
    k = k22(w,cf,0);
  
  fillm(0.0,d);
  
  d[0][0] = k;   d[0][1] = 0.0;
  d[1][0] = 0.0; d[1][1] = k;
}

/**
   function creates conductivity matrix of the material for 3D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/

void saltmat1::matcond3d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double w,cf;
  k = 0.0;

  w = Tm->ip[ipp].av[0];
  cf = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = k11(w,cf,0);
  if((ri == 0) && (ci == 1))
    k = k12(w,cf,0);
  if((ri == 1) && (ci == 0))
    k = k21(w,cf,0);
  if((ri == 1) && (ci == 1))
    k = k22(w,cf,0);
  
  fillm(0.0,d);
  
  d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0;
  d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0;
  d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;
}


/**
   function creates capacity matrix of the material

   @param c   - capacity matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void saltmat1::matcap (double &c,long ri,long ci,long ipp)
{
  double w,cf;
  c = 0.0;

  w = Tm->ip[ipp].av[0];
  cf = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    c = c11(w,cf,0);
  if((ri == 0) && (ci == 1))
    c = c12(w,cf,0);
  if((ri == 1) && (ci == 0))
    c = c21(w,cf,0);
  if((ri == 1) && (ci == 1))
    c = c22(w,cf,0);
}


void saltmat1::read (XFILE *in)
{
int i, j;
      for (i = 2; i<19; i++)
          {
          xfscanf(in, "%d",&MatChar[i]);
          }
     xfscanf(in, "%d",&MatChar[19]);
     for (i = 2;i <= 19; i++)
                         {

                         switch (MatChar[i])
                              {
                              case 0:
                              break;
                              case 1:xfscanf(in, "%lf",&MatConst[i]);
                                     //edit_chechbox(i,1);
                              break;
                              case 2:   if( MatChar[i-1] ==2)
                                             {
                                              xfscanf(in, "%lf %lf",&MatData[i][0][0],&MatData[i][1][0]);
                                              }
                                        else
                                             {
                                             xfscanf(in, "%lf %lf",&MatData[i][0][0],&MatData[i][1][0]);
                                             }

                                        switch (int(MatData[i][1][0]))
                                             {
                                             case 2: for (j=1; j<=MatData[i][0][0];j++)
                                                            {
                                                            xfscanf(in, "%lf %lf",&MatData[i][0][j],&MatData[i][1][j] );
                                                            }
                                             break;
                                             case 3: for (j=1; j<=MatData[i][0][0];j++)
                                                            {
                                                            xfscanf(in, "%lf %lf %lf",&MatData[i][0][j],&MatData[i][1][j],&MatData[i][2][j] );
                                                            }
                                             break;
                                             }
                                        xfscanf(in, "%d",&MatData[i][0][0]);
                                     
                              break;
                              case 3://edit_chechbox(i,3);
                              break;
                              case 30:   if( MatChar[i-1] ==1)
                                             {
                                               xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
                                              }
                                        else
                                             {
                                             xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
                                             }

                              break;
                              case 31: if( MatChar[i-1] ==1)
                                             {
                                              if (i == 6)
                                                  {
                                                   xfscanf(in, "%lf %lf %lf",&MatFunce[i][0],&MatFunce[i][1], &MatFunce[i][2]);
                                                  }
                                              else
                                                  {
                                                   xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
                                                  }
                                              }
                                        else
                                             {
                                             if (i == 6)
                                                  {
                                                   xfscanf(in, "%lf %lf %lf",&MatFunce[i][0],&MatFunce[i][1], &MatFunce[i][2]);
                                                  }
                                              else
                                                  {
                                                   xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
                                                  }
                                             }

                              break;
                              case 32:  if( MatChar[i-1] ==1)
                                             {
                                                xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
                                              }
                                        else
                                             {
                                               xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
                                             }
                              break;
							  default:{
									fprintf (stderr,"\n\n unknown definition of Material SalMat1 is required");
									fprintf (stderr,"\n in function read (file %s, line %d)\n",__FILE__,__LINE__);
									  }
                              }
                         }

}


/* Function calculates all auxiliary data - optional */
void saltmat1::auxiliarydata (double x1,double x2,double x3)
{}



/******************************************************
Conductivity and capacity terms for C and K matrices
*******************************************************/

double saltmat1::c11(double x1,double x2,double x3)
{
  double c11;

  c11 = 1.0;
  return (c11);
}

double saltmat1::c12(double x1,double x2,double x3)
{
  double c12;

   c12 = 0.0;
  return (c12);
}

double saltmat1::c13(double x1,double x2,double x3)
{
  double c13;

  c13 = 0.0;
  return (c13);
}

double saltmat1::c21(double x1,double x2,double x3)
{
  double c21;

  c21 = x2;
  return (c21);
}

double saltmat1::c22(double x1,double x2,double x3)
{
  double c22;
  double dCbDcf;
  int k;
  double a,b, Cb;

  if (x2 <= 0) x2 = 1e-10;
  if (x1 <0) x1 = 0;

  CorD(15,k,1,x2,a,b,a3);

  switch (k)
     {
     case 31: dCbDcf = b*a*pow(x2,(b-1));
     break;
     case 30:   Cb =  1/(1/(a*b*x2)+1/b);
                dCbDcf = Cb*Cb/(a*b*x2*x2);
     break;
     case 2: binding_izoterm_derivation(x2,dCbDcf);
     break;
     case 0: dCbDcf =0;// pouziti dat
     break;
     }

  c22 = x1 + dCbDcf;
  return (c22);
}

double saltmat1::c23(double x1,double x2,double x3)
{
  double c23;

  c23 = 0.0;
  return (c23);
}

double saltmat1::c31(double x1,double x2,double x3)
{
  double c31;

  c31 = 0.0;

  return (c31);
}

double saltmat1::c32(double x1,double x2,double x3)
{
  double c32;

  c32 = 0.0;
  return (c32);
}

double saltmat1::c33(double x1,double x2,double x3)
{
  double c33;

  c33 = 0.0;
  return (c33);
}

double saltmat1::k11(double x1,double x2,double x3)
{
  double k11;
  int k;
  double a,b;
  if (x1 <0)x1 = 0;
  CorD(5,k,1,x1,a,b,a3);

  switch (k)
     {
     case 0: k11 = 0;
     break;
     case 1: k11 = a;
     break;
     case 30: k11 = a*exp(b*x1);// exponenciala, zavislost kapy jen na vlhkosti
     break;
     case 2: k11 = a;
     break;
     }

  return(k11);
}

double saltmat1::k12(double x1,double x2,double x3)
{
  double k12;

  k12 = 0.0;
  return(k12);
}

double saltmat1::k13(double x1,double x2,double x3)
{
  double k13;

  k13 = 0.0;
  return(k13);
}

double saltmat1::k21(double x1,double x2,double x3)
{
  double k21;
  int k;
  double a,b;
  if (x1 <0)x1 = 0;

  CorD(5,k,1,x1,a,b,a3);

  switch (k)
     {
     case 0: k21 = 0;
     break;
     case 1: k21 = x2*a;
     break;
     case 30: k21 = x2*a*exp(b*x1);// exponenciala, zavislost kapy jen na vlhkosti
     break;
     case 2: k21 = x2*a;
     break;
     }


  return(k21);
}

double saltmat1::k22(double x1,double x2,double x3)
{
  double k22;
  int k;
  double a, b;
  if (x1 <0)x1 = 0;

  CorD(14,k,1,x2,a,b,a3);

  switch (k)
     {
     case 1: k22 = x1*a;
     break;
     case 30: k22 = x1*a*exp(b*x1);// exponenciala, zavislost kapy jen na vlhkosti
     break;
     case 2:  k22 = x1 * a;
     break;
     }



  return(k22);
}

double saltmat1::k23(double x1,double x2,double x3)
{
  double k23;

  k23 = 0.0;
  return(k23);
}

double saltmat1::k31(double x1,double x2,double x3)
{
  double k31;

  k31 = 0.0;
  return(k31);
}

double saltmat1::k32(double x1,double x2,double x3)
{
  double k32;

  k32 = 0.0;
  return(k32);
}

double saltmat1::k33(double x1,double x2,double x3)
{
  double k33;

   k33 = 0.0;
  return(k33);
}


// old function, it will be corrected!!??
/******************
Boundary conditions
*******************/
	
/**  moisture boundary conditions

   function creates correct new nodal value on the boundary (transmission) for 1st medium
   @param new_nodval - new prescribed value near the boundary
   @param bv         - value of prescribed value near the boundary
   @param x1 ... x3  - actual unknowns on the boundary
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
void saltmat1::get_fw_vec_nodval(double &new_nodval,double &bv,double x1,double x2,double x3,long bc,long ipp)
{
  double f11;

  switch (bc){//type of prescribed variable
  case 30:{//transmission
    f11 = bv;//should be changed
    break;
  }
  default:{
    fprintf (stderr,"\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__);
    exit(0);
  }
  }
  new_nodval = f11;
}


/**
   function creates correct transfer coefficient on the boundary (transmission) for 1st medium
   @param f11        - correct transfer coefficient
   @param x1 ... x3  - actual unknowns on the boundary
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
void saltmat1::get_fw_matrix(double &f11,double x1,double x2,double x3,long bc,long ipp)
{
  switch (bc){//type of prescribed variable
  case 30:{//transmission
    f11=1.0;//should be changed
    break;
  }
  default:{
    fprintf (stderr,"\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__);
    exit(0);
  }
  }
}


/**
   function creates flux on the boundary (transmission - convective mass transfer) for 1st medium
   @param new_nodval - flux on the boundary
   @param bv         - prescribed value near the boundary
   @param x1 ... x3  - actual unknowns on the boundary
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
void saltmat1::get_fw_flux(double &new_nodval,double &bv,double x1,double x2,double x3,long bc,long ipp)
{
  double f11;

  switch (bc){//type of prescribed variable
  case 30:{//transmission - boundary flux
    f11 = (bv - x1);//should be changed
    break;
  }
  default:{
    fprintf (stderr,"\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__);
    exit(0);
  }
  }
  new_nodval = f11;
}

//soils boundary conditions

	
/**
   function creates correct new nodal value on the boundary (transmission) for 1st medium
   @param new_nodval - new prescribed value near the boundary
   @param bv         - value of prescribed value near the boundary
   @param x1 ... x3  - actual unknowns on the boundary
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
void saltmat1::get_fcf_vec_nodval(double &new_nodval,double &bv,double x1,double x2,double x3,long bc,long ipp)
{
  double f11;

  switch (bc){//type of prescribed variable
  case 30:{//transmission
    f11 = bv;//should be changed
    break;
  } 
  default:{
    fprintf (stderr,"\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__);
    exit(0);
  }
  }
  new_nodval = f11;
}


/**
   function creates correct transfer coefficient on the boundary (transmission) for 1st medium
   @param f11        - correct transfer coefficient
   @param x1 ... x3  - actual unknowns on the boundary
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
void saltmat1::get_fcf_matrix(double &f11,double x1,double x2,double x3,long bc,long ipp)
{
  switch (bc){//type of prescribed variable
  case 30:{//transmission
    f11=1.0;//should be changed
    break;
  }
  default:{
    fprintf (stderr,"\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__);
    exit(0);
  }
  }
}


/**
   function creates flux on the boundary (transmission - convective mass transfer) for 1st medium
   @param new_nodval - flux on the boundary
   @param bv         - prescribed value near the boundary
   @param x1 ... x3  - actual unknowns on the boundary
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
void saltmat1::get_fcf_flux(double &new_nodval,double &bv,double x1,double x2,double x3,long bc,long ipp)
{
  double f11;

  switch (bc){//type of prescribed variable
  case 30:{//transmission - boundary flux
    f11 = (bv - x1);//should be changed
    break;
  }
  default:{
    fprintf (stderr,"\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__);
    exit(0);
  }
  }
  new_nodval = f11;
}


/**
   function computes all variables in nodes
   @param compother  - number of other components
   @param ipp        - first integration point on element
   @param x1 ... x3  - actual unknowns on the boundary
*/

double saltmat1::get_othervalue(long compother,long ipp, double x1,double x2,double x3)
{
  double other;

  switch (compother){
  case 0:{//first unknown
    other = x1;//should be changed
      break;
  }
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return (other);
}

/**
     function prints names of all variables in nodes
     @param out       - output file
     @param compother - number of other components
*/
void saltmat1::print_othervalue_name(FILE *out,long compother)
{
  switch (compother){
  case 0:{//first unknown
    fprintf (out,"First unknown (Units)        ");//should be changed
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}

void saltmat1::CorD(int cislochar, int &kvyhl,double in, double x, double & y, double & z, double &z2)
{

  switch (MatChar[cislochar])
     {
     case 0:
     break;
     case 1: y = MatConst[cislochar];
     break;
     case 2:   {
                int i = 1;

                int psl, prad;
                prad  = int (MatData[cislochar][0][0]);
                psl = int (MatData[cislochar][1][0]);
                if (x <=0)
                     {
                     x = 0;
                     y = MatData[cislochar][1][1];
                     if (psl == 3)
                         {
                          z = MatData[cislochar][2][1];
                          }

                     }
                else {
                      if (x > in)
                         {
                         x = in;
                         y = MatData[cislochar][1][prad];
                         if (psl == 3)
                              {
                              z = MatData[cislochar][2][prad];
                              }
                         }
                      else
                         {
                         while (x > MatData[cislochar][0][i]){
                              i++;
                              }

                         y = MatData[cislochar][1][i-1] + ((x - MatData[cislochar][0][i-1])*(MatData[cislochar][1][i] - MatData[cislochar][1][i-1]))/(MatData[cislochar][0][i] - MatData[cislochar][0][i-1]);
                         if (psl == 3)
                              {
                              z = MatData[cislochar][2][i-1] + ((x - MatData[cislochar][0][i-1])*(MatData[cislochar][2][i] - MatData[cislochar][2][i-1]))/(MatData[cislochar][0][i] - MatData[cislochar][0][i-1]);
                              }
                         }
                      }
                kvyhl = 2;
                }
     break;
     case 30:  y = MatFunce[cislochar][0];
               z = MatFunce[cislochar][1];
               kvyhl = 30;
     break;
     case 31:  y = MatFunce[cislochar][0];
               z = MatFunce[cislochar][1];
               if (cislochar == 6)
                    {
                    z2 = MatFunce[cislochar][2];
                    }
               kvyhl = 31;
     break;
     case 32:  y = MatFunce[cislochar][0];
               z = MatFunce[cislochar][1];
               kvyhl = 32;
     break;
     case 33: //zatim nic
     break;
	 default:{
			fprintf (stderr,"\n\n unknown definition of material parameter is required");
			fprintf (stderr,"\n in function CorD (file %s, line %d)\n",__FILE__,__LINE__);
			}
     }

}

void saltmat1::binding_izoterm_derivation(double x2, double deribi)
{
int i = 0;
if ( x2 < 0)
     {
     x2 = 0;
      }

 while (x2 > MatData[15][0][i]){
                              i++;
                              }
 deribi = (MatData [15][1][i] - MatData [15][1][i-1])/(MatData [15][0][i] - MatData [15][0][i-1]);
     //TODO: Add your source code here
}
